@@ -647,46 +647,46 @@ function version_exceeds(version, min_version) {
 const DESKTOP_ACTION_ICON_NAMES = {
     area_shot: 'screenshot-area',
     base: 'x-office-database',
-    big_picture: 'view-fullscreen',
-    calc: 'x-office-spreadsheet',
-    community: 'system-users',
-    compose: 'text-editor',
-    contacts: 'x-office-address-book',
-    document: 'document-new',
-    draw: 'x-office-drawing',
-    friends: 'user-available',
-    fullscreen: 'view-fullscreen',
-    impress: 'x-office-presentation',
-    library: 'accessories-dictionary',
+    big_picture: 'xsi-view-fullscreen-symbolic',
+    calc: 'xsi-x-office-spreadsheet-symbolic',
+    community: 'xsi-users-symbolic',
+    compose: 'xsi-text-editor-symbolic',
+    contacts: 'xsi-x-office-address-book-symbolic',
+    document: 'xsi-document-new-symbolic',
+    draw: 'xsi-x-office-drawing-symbolic',
+    friends: 'xsi-user-available-symbolic',
+    fullscreen: 'xsi-view-fullscreen-symbolic',
+    impress: 'xsi-x-office-presentation-symbolic',
+    library: 'xsi-dictionary-symbolic',
     math: 'x-office-math',
-    mute: 'audio-volume-muted',
-    new_document: 'document-new',
+    mute: 'xsi-audio-volume-muted-symbolic',
+    new_document: 'xsi-document-new-symbolic',
     new_private_window: 'view-private',
-    new_root_window: 'dialog-password',
+    new_root_window: 'xsi-dialog-password-symbolic',
     news: 'news',
     new_session: 'xsi-tab-new-symbolic',
-    new_window: 'window-new',
-    next: 'media-skip-forward',
-    open_computer: 'computer',
-    open_home: 'user-home',
-    open_trash: 'user-trash',
-    play: 'media-playback-start',
-    play_pause: 'media-playback-start',
-    preferences: 'preferences-other',
-    prefs: 'preferences-other',
-    previous: 'media-skip-backward',
+    new_window: 'xsi-window-new-symbolic',
+    next: 'xsi-media-skip-forward-symbolic',
+    open_computer: 'xsi-computer-symbolic',
+    open_home: 'xsi-user-home-symbolic',
+    open_trash: 'xsi-user-trash-symbolic',
+    play: 'xsi-media-playback-start-symbolic',
+    play_pause: 'xsi-media-playback-start-symbolic',
+    preferences: 'xsi-preferences-symbolic',
+    prefs: 'xsi-preferences-symbolic',
+    previous: 'xsi-media-skip-backward-symbolic',
     screen_shot: 'screenshot-fullscreen',
-    screenshots: 'applets-screenshooter',
-    servers: 'network-server',
-    settings: 'preferences-other',
+    screenshots: 'xsi-screenshooter-symbolic',
+    servers: 'xsi-network-server-symbolic',
+    settings: 'xsi-preferences-symbolic',
     ssa: 'screenshot-area',
     ssf: 'screenshot-fullscreen',
     ssw: 'screenshot-window',
-    stop_quit: 'media-playback-stop',
+    stop_quit: 'xsi-media-playback-stop-symbolic',
     store: 'store',
-    window: 'window-new',
+    window: 'xsi-window-new-symbolic',
     window_shot: 'screenshot-window',
-    writer: 'x-office-document',
+    writer: 'xsi-x-office-document-symbolic',
 };
 
 /**
